import { Directive, TemplateRef, Input, Component, ContentChild, ɵɵdefineInjectable, Injectable, EventEmitter, ContentChildren, Output, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FlexLayoutModule } from '@angular/flex-layout';
import { FormControl, ReactiveFormsModule } from '@angular/forms';
import { takeUntil } from 'rxjs/operators';
import { Subject } from 'rxjs';

class Sort {
}
var SortDirection;
(function (SortDirection) {
    SortDirection["ASC"] = "ASC";
    SortDirection["DESC"] = "DESC";
})(SortDirection || (SortDirection = {}));

class NguDatatableCell {
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
}
NguDatatableCell.decorators = [
    { type: Directive, args: [{
                selector: "[nguDatatableCell]"
            },] }
];
NguDatatableCell.ctorParameters = () => [
    { type: TemplateRef }
];
NguDatatableCell.propDecorators = {
    nguDatatableCell: [{ type: Input, args: ['nguDatatableCell',] }]
};

class NguDatatableHeader {
    constructor(templateRef) {
        this.templateRef = templateRef;
    }
}
NguDatatableHeader.decorators = [
    { type: Directive, args: [{
                selector: '[nguDatatableHeader]'
            },] }
];
NguDatatableHeader.ctorParameters = () => [
    { type: TemplateRef }
];

class NguDatatableColumn {
    ngOnInit() {
        if (!this.property) {
            throw new TypeError("'Property' attribute is required for ngu-datatable-column. Example: <ngu-datatable-column property=\"id\">");
        }
    }
}
NguDatatableColumn.decorators = [
    { type: Component, args: [{
                selector: "ngu-datatable-column",
                template: "",
                styles: [""]
            },] }
];
NguDatatableColumn.propDecorators = {
    property: [{ type: Input, args: ["property",] }],
    sort: [{ type: Input, args: ["sort",] }],
    nguDatatableCell: [{ type: ContentChild, args: [NguDatatableCell,] }],
    nguDatatableHeader: [{ type: ContentChild, args: [NguDatatableHeader,] }]
};

class NguDatatablePaginationService {
    constructor() {
        this.pagination = {
            currentPage: 1,
            pageSize: 10,
        };
    }
}
NguDatatablePaginationService.ɵprov = ɵɵdefineInjectable({ factory: function NguDatatablePaginationService_Factory() { return new NguDatatablePaginationService(); }, token: NguDatatablePaginationService, providedIn: "root" });
NguDatatablePaginationService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
NguDatatablePaginationService.ctorParameters = () => [];

class NguDatatableComponent {
    constructor(paginationService) {
        this.paginationService = paginationService;
        this.context = {
            data: [],
            visibleData: [],
            selectedData: [],
            sort: null,
        };
        this.SortDirection = SortDirection;
        this.selectAllEnabled = false;
        this.rowClick = new EventEmitter();
    }
    set data(dataSource) {
        this.context.data = dataSource;
        this.setVisibleData();
    }
    get data() {
        return this.context.data;
    }
    onPaginationChange() {
        this.setVisibleData();
    }
    setVisibleData() {
        const pageSize = this.paginationService.pagination.pageSize;
        const currentPage = this.paginationService.pagination.currentPage;
        const startValue = (currentPage - 1) * pageSize;
        this.context.visibleData = this.context.data.slice(startValue, startValue + pageSize);
    }
    selectAllData(checked) {
        if (checked) {
            this.context.selectedData = this.context.visibleData.slice();
        }
        else {
            this.context.selectedData = [];
        }
    }
    selectRow(checked, row) {
        if (checked) {
            this.context.selectedData.push(row);
        }
        else {
            this.context.selectedData.splice(this.context.selectedData.indexOf(row), 1);
        }
    }
    isRowSelected(row) {
        return this.context.selectedData.includes(row);
    }
    onSortClick(column) {
        if (!!column.sort || column.sort == "") {
            if (!!this.context.sort) {
                this.context.sort = {
                    column: column.property,
                    direction: this.context.sort.column === column.property &&
                        this.context.sort.direction === SortDirection.DESC
                        ? SortDirection.ASC
                        : SortDirection.DESC,
                };
            }
            else {
                this.context.sort = {
                    column: column.property,
                    direction: SortDirection.DESC,
                };
            }
            this.context.data = this.data.sort((a, b) => {
                if (this.context.sort.direction === SortDirection.DESC) {
                    return a[column.property] < b[column.property]
                        ? 1
                        : a[column.property] > b[column.property]
                            ? -1
                            : 0;
                }
                else {
                    return a[column.property] > b[column.property]
                        ? 1
                        : a[column.property] < b[column.property]
                            ? -1
                            : 0;
                }
            });
            this.setVisibleData();
        }
    }
    getSortArrow(column) {
        if (!this.context.sort) {
            return "";
        }
        else if (this.context.sort.column === column.property) {
            return this.context.sort.direction === SortDirection.DESC
                ? "headerSortDown"
                : "headerSortUp";
        }
    }
}
NguDatatableComponent.decorators = [
    { type: Component, args: [{
                selector: "ngu-datatable",
                template: "<div fxLayout=\"column\" fxFlex=\"100\" class=\"ngu-datatable-container\">\r\n  <div class=\"ngu-datatable-header-row\">\r\n    <ng-container *ngIf=\"selectAllEnabled\">\r\n      <div class=\"select-all-header-checkbox\">\r\n        <input\r\n          type=\"checkbox\"\r\n          (change)=\"selectAllData($event.target.checked)\"\r\n        /></div\r\n    ></ng-container>\r\n    <ng-container *ngFor=\"let column of columns\">\r\n      <div\r\n        fxLayout=\"row\"\r\n        class=\"ngu-datatable-header-cell\"\r\n        (click)=\"onSortClick(column)\"\r\n        [fxFlex]=\"100 / columns.length\"\r\n      >\r\n        <ng-container *ngIf=\"column.nguDatatableHeader; else defaultHeader\">\r\n          <ng-container\r\n            *ngTemplateOutlet=\"\r\n              column?.nguDatatableHeader?.templateRef;\r\n              context: { $implicit: 'row' }\r\n            \"\r\n          ></ng-container>\r\n        </ng-container>\r\n        <ng-template #defaultHeader>\r\n          {{ column.property }}\r\n        </ng-template>\r\n        <ng-container *ngIf=\"column.sort || column.sort === ''\"\r\n          ><div\r\n            [ngClass]=\"getSortArrow(column)\"\r\n          ></div\r\n        ></ng-container>\r\n      </div>\r\n    </ng-container>\r\n  </div>\r\n\r\n  <ng-container *ngFor=\"let row of context.visibleData\">\r\n    <div (click)=\"rowClick.emit(row)\" fxFlex=\"100\" class=\"ngu-datatable-data-row\">\r\n      <ng-container *ngIf=\"selectAllEnabled\">\r\n        <div class=\"select-row-checkbox\">\r\n          <input\r\n            type=\"checkbox\"\r\n            (change)=\"selectRow($event.target.checked, row)\"\r\n            [checked]=\"isRowSelected(row)\"\r\n          />\r\n        </div>\r\n      </ng-container>\r\n      <ng-container *ngFor=\"let column of columns\">\r\n        <div [fxFlex]=\"100 / columns.length\" class=\"ngu-datatable-data-cell\">\r\n          <ng-container\r\n            *ngTemplateOutlet=\"\r\n              column?.nguDatatableCell?.templateRef;\r\n              context: { $implicit: row }\r\n            \"\r\n          ></ng-container>\r\n        </div>\r\n      </ng-container>\r\n    </div>\r\n  </ng-container>\r\n  <div fxFlex=\"100\" class=\"ngu-datatable-pagination\">\r\n    <ngu-datatable-pagination\r\n      fxFlex=\"100\"\r\n      [entries]=\"context.data.length\"\r\n      (paginationChange)=\"onPaginationChange()\"\r\n    ></ngu-datatable-pagination>\r\n  </div>\r\n</div>\r\n",
                styles: [".ngu-datatable-container{border-collapse:collapse;box-shadow:0 4px 8px 0 rgba(0,0,0,.2);display:flex;flex-direction:column}.ngu-datatable-container .ngu-datatable-header-row{background-color:#0e0e4e;border:1px solid #ddd;display:flex;padding:8px}.ngu-datatable-container .ngu-datatable-header-row .ngu-datatable-header-cell{color:#fff}.ngu-datatable-container .ngu-datatable-header-row .select-all-header-checkbox{padding-right:.5rem}.ngu-datatable-container .ngu-datatable-data-row{border:1px solid #ddd;display:flex;padding:8px}.ngu-datatable-container .ngu-datatable-data-row .select-row-checkbox{padding-right:.5rem}.ngu-datatable-container .ngu-datatable-data-row:hover,.ngu-datatable-container .ngu-datatable-data-row:nth-child(odd):hover{background-color:#c6c6e6;transition:.2s}.ngu-datatable-container .ngu-datatable-data-row:nth-child(odd){background-color:#f2f2f2}.ngu-datatable-container .ngu-datatable-pagination{border:1px solid #ddd;display:flex;padding:8px}.headerSortDown:after,.headerSortUp:after{border:8px solid transparent;content:\" \";left:2px;position:relative}.headerSortDown:after{border-top-color:#fff;top:10px}.headerSortUp:after{border-bottom-color:#fff;bottom:15px}.headerSortDown,.headerSortUp{padding-right:10px}"]
            },] }
];
NguDatatableComponent.ctorParameters = () => [
    { type: NguDatatablePaginationService }
];
NguDatatableComponent.propDecorators = {
    data: [{ type: Input, args: ["data",] }],
    selectAllEnabled: [{ type: Input, args: ["selectAllEnabled",] }],
    columns: [{ type: ContentChildren, args: [NguDatatableColumn, { descendants: true },] }],
    rowClick: [{ type: Output }]
};

class NguDatatablePaginationComponent {
    constructor(paginationService) {
        this.paginationService = paginationService;
        this.currentPageSize = 10;
        this.currentPage = 1;
        this.paginationChange = new EventEmitter();
        this.possiblePageSize = [10, 30, 50];
        this.selectFormControl = new FormControl(this.currentPageSize);
        this.destroy$ = new Subject();
    }
    ngOnInit() {
        this.onChange();
        this.selectFormControl.valueChanges
            .pipe(takeUntil(this.destroy$))
            .subscribe(value => {
            this.currentPageSize = value;
            this.currentPage = 1;
            this.onChange();
        });
    }
    getFirstRowBasedOnPage() {
        if (this.entries) {
            return 1 + (this.currentPage - 1) * this.currentPageSize;
        }
        return 0;
    }
    getLastRowBasedOnPage() {
        if (this.entries) {
            if (this.currentPage * this.currentPageSize > this.entries) {
                return this.entries;
            }
            else {
                return this.currentPage * this.currentPageSize;
            }
        }
        return 0;
    }
    decreaseCurrentPage() {
        if (this.currentPage > 1) {
            this.currentPage--;
            this.onChange();
        }
    }
    onChange() {
        this.paginationService.pagination.pageSize = this.currentPageSize;
        this.paginationService.pagination.currentPage = this.currentPage;
        this.paginationChange.emit();
    }
    increaseCurrentPage() {
        if (this.currentPage !== this.getMaxPages()) {
            this.currentPage++;
            this.onChange();
        }
    }
    getMaxPages() {
        if (this.entries && this.currentPageSize < this.entries) {
            return Math.ceil(this.entries / this.currentPageSize);
        }
        return 1;
    }
    getPages() {
        const pages = [];
        for (let i = 1; i < this.getMaxPages() + 1; i++) {
            pages.push(i);
        }
        return pages;
    }
    isPageVisible(page) {
        if ((page === 4 || page === 3) && this.currentPage === 1) {
            return true;
        }
        const difference = page - this.currentPage;
        return difference < 3 && difference > -3;
    }
}
NguDatatablePaginationComponent.decorators = [
    { type: Component, args: [{
                selector: "ngu-datatable-pagination",
                template: "<div fxFlex=\"100\" fxLayout=\"row\" class=\"pagination-container\">\r\n  <div fxFlex=\"40\"></div>\r\n  <div fxFlex=\"60\" class=\"pagination-info\" fxLayout=\"row\" fxLayoutAlign=\"flex-end\">\r\n    <div class=\"row-selection-container\" fxLayout=\"row\">\r\n      <span>Records per page:</span>\r\n      <select class=\"row-selection\" [formControl]=\"selectFormControl\">\r\n        <option\r\n          [ngValue]=\"pageSize\"\r\n          *ngFor=\"let pageSize of possiblePageSize\"\r\n          >{{ pageSize }}</option\r\n        >\r\n      </select>\r\n    </div>\r\n    <span class=\"entries\"\r\n      >{{ getFirstRowBasedOnPage() }}-{{ getLastRowBasedOnPage() }} of\r\n      {{ entries }}</span\r\n    >\r\n    <nav role=\"navigation\" class=\"pagination-box\">\r\n      <ul class=\"pagination\">\r\n        <li> <span (click)=\"decreaseCurrentPage()\"> < </span></li>\r\n        <li> <span (click)=\"increaseCurrentPage()\"> > </span> </li>\r\n      </ul>\r\n    </nav>\r\n  </div>\r\n</div>\r\n",
                styles: [".pagination-container .pagination-info{align-items:center!important}.pagination-container .pagination-info ul.pagination{display:inline-block;margin:0;padding:0}.pagination-container .pagination-info ul.pagination li{display:inline}.pagination-container .pagination-info ul.pagination li span{border:1px solid #000;color:#000;float:left;padding:8px 16px;text-decoration:none}.pagination-container .pagination-info .pagination li:first-child span{border-bottom-left-radius:5px;border-top-left-radius:5px}.pagination-container .pagination-info .pagination li:last-child span{border-bottom-right-radius:5px;border-top-right-radius:5px}.pagination-container .pagination-info ul.pagination li span.active{background-color:#c0bcf2;border:1px solid grey;color:#fff}.pagination-container .pagination-info ul.pagination li span:hover:not(.active){background-color:#8e8af2}.pagination-container .pagination-info .pagination-box,.pagination-container .pagination-info .row-selection-container{margin:0 .5rem}"]
            },] }
];
NguDatatablePaginationComponent.ctorParameters = () => [
    { type: NguDatatablePaginationService }
];
NguDatatablePaginationComponent.propDecorators = {
    entries: [{ type: Input }],
    currentPageSize: [{ type: Input }],
    currentPage: [{ type: Input }],
    paginationChange: [{ type: Output }],
    possiblePageSize: [{ type: Input }]
};

class NguDatatableModule {
}
NguDatatableModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NguDatatableComponent, NguDatatableColumn, NguDatatableCell, NguDatatableHeader, NguDatatablePaginationComponent],
                imports: [CommonModule, FlexLayoutModule, ReactiveFormsModule],
                exports: [NguDatatableComponent, NguDatatableColumn, NguDatatableCell, NguDatatableHeader]
            },] }
];

/*
 * Public API Surface of ngu-datatable
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NguDatatableCell, NguDatatableComponent, NguDatatableModule, NguDatatableColumn as ɵa, NguDatatableHeader as ɵb, NguDatatablePaginationService as ɵc, NguDatatablePaginationComponent as ɵd };
//# sourceMappingURL=ngu-datatable.js.map
